@@ -23,7 +23,7 @@
  * SOFTWARE. */
 
 #include "includes.h"
-#include "util.h"
+#include "dbutil.h"
 #include "algo.h"
 #include "buffer.h"
 #include "session.h"
@@ -46,19 +46,26 @@ const unsigned char dh_p_val[] = {
 	0xEE, 0x38, 0x6B, 0xFB, 0x5A, 0x89, 0x9F, 0xA5, 0xAE, 0x9F, 0x24, 0x11,
 	0x7C, 0x4B, 0x1F, 0xE6, 0x49, 0x28, 0x66, 0x51, 0xEC, 0xE6, 0x53, 0x81,
 	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
-
 #define DH_P_LEN 128
-const unsigned int dh_p_digits = 310;
 
 const unsigned int dh_g_val = 2;
 
 static void read_kex();
 static void gen_new_keys();
+#ifndef DISABLE_ZLIB
+static void gen_new_zstreams();
+#endif
 /* helper function for gen_new_keys */
 static void hashkeys(unsigned char *out, int outlen, 
 		const hash_state * hs, unsigned const char X);
 static void send_msg_kexdh_reply(mp_int *dh_e);
 
+/* Executed upon receiving a kexinit message from the client to initiate
+ * key exchange. If we haven't already done so, we send the list of our
+ * preferred algorithms. The client's requested algorithms are processed,
+ * and we calculate the first portion of the key-exchange-hash for used
+ * later in the key exchange. No response is sent, as the client should
+ * initiate the diffie-hellman key exchange */
 void recv_msg_kexinit() {
 	
 	TRACE(("enter recv_msg_kexinit"));
@@ -100,6 +107,7 @@ void recv_msg_kexinit() {
 	TRACE(("leave recv_msg_kexinit"));
 }
 
+/* Bring new keys into use after a key exchange, and let the client know*/
 void send_msg_newkeys() {
 
 	TRACE(("enter send_msg_newkeys"));
@@ -121,6 +129,7 @@ void send_msg_newkeys() {
 	TRACE(("leave send_msg_newkeys"));
 }
 
+/* Bring the new keys into use after a key exchange */
 void recv_msg_newkeys() {
 
 	TRACE(("enter recv_msg_newkeys"));
@@ -138,7 +147,7 @@ void recv_msg_newkeys() {
 	TRACE(("leave recv_msg_newkeys"));
 }
 
-/* set the kex state variables to initial values, ready to receive a new
+/* Set the kex state variables to initial values, ready to receive a new
  * SSH_MSG_KEXINIT (or send one) */
 void kexinitialise() {
 
@@ -153,7 +162,6 @@ void kexinitialise() {
 	ses.kexstate.sentnewkeys = 0;
 
 	/* first_packet_follows */
-	/* TODO - currently not handled */
 	ses.kexstate.firstfollows = 0;
 
 	ses.kexstate.datatrans = 0;
@@ -166,12 +174,14 @@ void kexinitialise() {
 
 }
 
-/* helper function for gen_new_keys, creates a hash. It makes a copy of the
+/* Helper function for gen_new_keys, creates a hash. It makes a copy of the
  * already initialised hash_state hs, which should already have processed
  * the dh_K and hash, since these are common. X is the letter 'A', 'B' etc.
  * out must have at least min(SHA1_HASH_SIZE, outlen) bytes allocated.
  * The output will only be expanded once, since that is all that is required
- * (for 3DES and SHA, with 24 and 20 bytes respectively). */
+ * (for 3DES and SHA, with 24 and 20 bytes respectively). 
+ *
+ * See Section 5.2 of the IETF secsh Transport Draft for details */
 static void hashkeys(unsigned char *out, int outlen, 
 		const hash_state * hs, const unsigned char X) {
 
@@ -191,8 +201,12 @@ static void hashkeys(unsigned char *out, int outlen,
 	}
 }
 
-/* generate the actual encryption/integrity keys, using the results of the
- * key exchange */
+/* Generate the actual encryption/integrity keys, using the results of the
+ * key exchange, as specified in section 5.2 of the IETF secsh-transport
+ * draft. This occurs after the DH key-exchange.
+ *
+ * ses.newkeys is the new set of keys which are generated, these are only
+ * taken into use after both sides have sent a newkeys message */
 static void gen_new_keys() {
 
 	unsigned char IV[MAX_IV_LEN];
@@ -242,15 +256,27 @@ static void gen_new_keys() {
 	keysize = ses.newkeys->trans_algo_mac->keysize;
 	hashkeys(ses.newkeys->transmackey, keysize, &hs, 'F');
 
-	/* compression */
 #ifndef DISABLE_ZLIB
-	/* create new zstreams */
-	TRACE(("enter ZLIB section of genkeys"));
+	gen_new_zstreams();
+#endif
+	
+	/* Switch over to the new keys */
+	m_burn(ses.keys, sizeof(struct key_context));
+	m_free(ses.keys);
+	ses.keys = ses.newkeys;
+	ses.newkeys = NULL;
 
+	TRACE(("leave gen_new_keys"));
+}
+
+#ifndef DISABLE_ZLIB
+/* Set up new zlib compression streams, close the old ones. Only
+ * called from gen_new_keys() */
+static void gen_new_zstreams() {
+
+	/* create new zstreams */
 	if (ses.newkeys->recv_algo_comp == DROPBEAR_COMP_ZLIB) {
 		ses.newkeys->recv_zstream = (z_streamp)m_malloc(sizeof(z_stream));
-		ses.newkeys->recv_zstream->next_in = Z_NULL;
-		ses.newkeys->recv_zstream->avail_in = 0;
 		ses.newkeys->recv_zstream->zalloc = Z_NULL;
 		ses.newkeys->recv_zstream->zfree = Z_NULL;
 		
@@ -260,12 +286,9 @@ static void gen_new_keys() {
 	} else {
 		ses.newkeys->recv_zstream = NULL;
 	}
-	TRACE(("past inflateInit"));
 
 	if (ses.newkeys->trans_algo_comp == DROPBEAR_COMP_ZLIB) {
 		ses.newkeys->trans_zstream = (z_streamp)m_malloc(sizeof(z_stream));
-		ses.newkeys->trans_zstream->next_in = Z_NULL;
-		ses.newkeys->trans_zstream->avail_in = 0;
 		ses.newkeys->trans_zstream->zalloc = Z_NULL;
 		ses.newkeys->trans_zstream->zfree = Z_NULL;
 	
@@ -276,41 +299,29 @@ static void gen_new_keys() {
 	} else {
 		ses.newkeys->trans_zstream = NULL;
 	}
-	TRACE(("past deflateInit"));
-
 	
 	/* clean up old keys */
 	if (ses.keys->recv_zstream != NULL) {
-		int ret;
-		ret = inflateEnd(ses.keys->recv_zstream);
-		if (ret == Z_STREAM_ERROR) {
+		if (inflateEnd(ses.keys->recv_zstream) == Z_STREAM_ERROR) {
 			/* Z_DATA_ERROR is ok, just means that stream isn't ended */
 			dropbear_exit("crypto error");
 		}
 		m_free(ses.keys->recv_zstream);
 	}
 	if (ses.keys->trans_zstream != NULL) {
-		int ret;
-		ret = deflateEnd(ses.keys->trans_zstream);
-		/* Z_DATA_ERROR is ok, just means that stream isn't ended */
-		if (ret == Z_STREAM_ERROR) {
+		if (deflateEnd(ses.keys->trans_zstream) == Z_STREAM_ERROR) {
+			/* Z_DATA_ERROR is ok, just means that stream isn't ended */
 			dropbear_exit("crypto error");
 		}
 		m_free(ses.keys->trans_zstream);
 	}
-	TRACE(("leave ZLIB section of genkeys"));
+}
 #endif
 
-	
-	/* change the keys */
-	m_burn((void*)ses.keys, sizeof(struct key_context));
-	m_free(ses.keys);
-	ses.keys = ses.newkeys;
-	ses.newkeys = NULL;
-
-	TRACE(("leave gen_new_keys"));
-}
-
+/* Handle a diffie-hellman key exchange initialisation. This involves
+ * calculating a session key reply value, and corresponding hash. These
+ * are carried out by send_msg_kexdh_reply(). recv_msg_kexdh_init() calls
+ * that function, then brings the new keys into use */
 void recv_msg_kexdh_init() {
 
 	mp_int dh_e;
@@ -332,7 +343,12 @@ void recv_msg_kexdh_init() {
 	TRACE(("leave recv_msg_kexdh_init"));
 }
 	
-/* some mp_ints are cleared before the end to attempt to minimise memory */
+/* Generate our side of the diffie-hellman key exchange value (dh_f), and
+ * calculate the session key using the diffie-hellman algorithm. Following
+ * that, the session hash is calculated, and signed with RSA or DSS. The
+ * result is sent to the client. 
+ *
+ * See the ietf-secsh-transport draft, section 6, for details */
 static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	mp_int dh_p, dh_q, dh_g, dh_y, dh_f;
@@ -342,14 +358,7 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	TRACE(("enter send_msg_kexdh_reply"));
 	
-	assert(ses.kexstate.recvkexinit);
-
-	m_mp_init(&dh_g);
-	/*mp_set_prec(dh_p_digits);*/ /* we may as well set the size right to start */
-	m_mp_init(&dh_p);
-	m_mp_init(&dh_q);
-	m_mp_init(&dh_y);
-	m_mp_init(&dh_f);
+	m_mp_init_multi(&dh_g, &dh_p, &dh_q, &dh_y, &dh_f, NULL);
 
 	/* read the prime and generator*/
 	if (mp_read_unsigned_bin(&dh_p, (unsigned char*)dh_p_val, DH_P_LEN)
@@ -373,12 +382,12 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	/* calculate our random value dh_y */
 	do {
-		assert(dh_q_len <= sizeof(randbuf));
+		assert((unsigned int)dh_q_len <= sizeof(randbuf));
 		genrandom(randbuf, dh_q_len);
 		if (mp_read_unsigned_bin(&dh_y, randbuf, dh_q_len) != MP_OKAY) {
 			dropbear_exit("Diffie-Hellman error");
 		}
-	} while (mp_cmp(&dh_y, &dh_q) >= 0 || mp_cmp_d(&dh_y, 0) <= 0);
+	} while (mp_cmp(&dh_y, &dh_q) == MP_GT || mp_cmp_d(&dh_y, 0) != MP_GT);
 
 	/* f = g^y mod p */
 	if (mp_exptmod(&dh_g, &dh_y, &dh_p, &dh_f) != MP_OKAY) {
@@ -394,9 +403,7 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	}
 
 	/* clear no longer needed vars */
-	mp_clear(&dh_y);
-	mp_clear(&dh_p);
-	mp_clear(&dh_q);
+	mp_clear_multi(&dh_y, &dh_p, &dh_q, NULL);
 
 	/* Create the remainder of the hash buffer, to generate the exchange hash */
 	/* K_S, the host key */
@@ -409,33 +416,17 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	/* K, the shared secret */
 	buf_putmpint(ses.kexhashbuf, ses.dh_K);
 
-	/* calculate the hash H */
+	/* calculate the hash H to sign */
 	sha1_init(&hs);
 	buf_setpos(ses.kexhashbuf, 0);
 	sha1_process(&hs, buf_getptr(ses.kexhashbuf, ses.kexhashbuf->len),
 			ses.kexhashbuf->len);
 	sha1_done(&hs, ses.hash);
-#if defined(DEBUG_KEXHASH) && defined(DEBUG_TRACE)
-	{
-		int hashfd;
-		int hashwritelen;
-		hashfd = creat("/tmp/hashbufm", S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP);
-	
-		hashwritelen = write(hashfd, 
-				buf_getptr(ses.kexhashbuf, ses.kexhashbuf->len),
-				ses.kexhashbuf->len);
-		if (hashwritelen != ses.kexhashbuf->len) {
-			dropbear_exit("bad hashwritelen debugging");
-		}
-		close(hashfd);
-		printhex(ses.hash, SHA1_HASH_SIZE);
-	}
-#endif /* DEBUG_KEXHASH */
 	buf_free(ses.kexhashbuf);
 	ses.kexhashbuf = NULL;
 	
 	/* first time around, we set the session_id to H */
-	if (!ses.session_id) {
+	if (ses.session_id == NULL) {
 		/* create the session_id, this never needs freeing */
 		ses.session_id = (unsigned char*)m_malloc(SHA1_HASH_SIZE);
 		memcpy(ses.session_id, ses.hash, SHA1_HASH_SIZE);
@@ -468,12 +459,17 @@ static void read_kex() {
 	unsigned char* str;
 	char * erralgo = NULL;
 
+	int goodguess = 0;
+	int allgood = 1; /* we AND this with each goodguess and see if its still
+						true after */
+
 	buf_incrpos(ses.payload, 16); /* start after the cookie */
 
 	ses.newkeys = (struct key_context*)m_malloc(sizeof(struct key_context));
 
 	/* kex_algorithms */
-	algo = buf_match_algo(ses.payload, sshkex);
+	algo = buf_match_algo(ses.payload, sshkex, &goodguess);
+	allgood &= goodguess;
 	if (algo == NULL) {
 		erralgo = "kex";
 		goto error;
@@ -481,7 +477,8 @@ static void read_kex() {
 	ses.newkeys->algo_kex = algo->val;
 
 	/* server_host_key_algorithms */
-	algo = buf_match_algo(ses.payload, sshhostkey);
+	algo = buf_match_algo(ses.payload, sshhostkey, &goodguess);
+	allgood &= goodguess;
 	if (algo == NULL) {
 		erralgo = "hostkey";
 		goto error;
@@ -489,7 +486,7 @@ static void read_kex() {
 	ses.newkeys->algo_hostkey = algo->val;
 
 	/* encryption_algorithms_client_to_server */
-	algo = buf_match_algo(ses.payload, sshciphers);
+	algo = buf_match_algo(ses.payload, sshciphers, &goodguess);
 	if (algo == NULL) {
 		erralgo = "enc c->s";
 		goto error;
@@ -497,7 +494,7 @@ static void read_kex() {
 	ses.newkeys->recv_algo_crypt = (struct dropbear_cipher*)algo->data;
 
 	/* encryption_algorithms_server_to_client */
-	algo = buf_match_algo(ses.payload, sshciphers);
+	algo = buf_match_algo(ses.payload, sshciphers, &goodguess);
 	if (algo == NULL) {
 		erralgo = "enc s->c";
 		goto error;
@@ -505,7 +502,7 @@ static void read_kex() {
 	ses.newkeys->trans_algo_crypt = (struct dropbear_cipher*)algo->data;
 
 	/* mac_algorithms_client_to_server */
-	algo = buf_match_algo(ses.payload, sshhashes);
+	algo = buf_match_algo(ses.payload, sshhashes, &goodguess);
 	if (algo == NULL) {
 		erralgo = "mac c->s";
 		goto error;
@@ -513,7 +510,7 @@ static void read_kex() {
 	ses.newkeys->recv_algo_mac = (struct dropbear_hash*)algo->data;
 
 	/* mac_algorithms_server_to_client */
-	algo = buf_match_algo(ses.payload, sshhashes);
+	algo = buf_match_algo(ses.payload, sshhashes, &goodguess);
 	if (algo == NULL) {
 		erralgo = "mac s->c";
 		goto error;
@@ -521,7 +518,7 @@ static void read_kex() {
 	ses.newkeys->trans_algo_mac = (struct dropbear_hash*)algo->data;
 
 	/* compression_algorithms_client_to_server */
-	algo = buf_match_algo(ses.payload, sshcompress);
+	algo = buf_match_algo(ses.payload, sshcompress, &goodguess);
 	if (algo == NULL) {
 		erralgo = "comp c->s";
 		goto error;
@@ -529,7 +526,7 @@ static void read_kex() {
 	ses.newkeys->recv_algo_comp = algo->val;
 
 	/* compression_algorithms_server_to_client */
-	algo = buf_match_algo(ses.payload, sshcompress);
+	algo = buf_match_algo(ses.payload, sshcompress, &goodguess);
 	if (algo == NULL) {
 		erralgo = "comp s->c";
 		goto error;
@@ -547,19 +544,21 @@ static void read_kex() {
 	/* first_kex_packet_follows */
 	if (buf_getbyte(ses.payload)) {
 		ses.kexstate.firstfollows = 1;
-		/* XXX currently not handled */
+		/* if the guess wasn't good, we ignore the packet sent */
+		if (!allgood) {
+			ses.ignorenext = 1;
+		}
 	}
 
 	/* reserved for future extensions */
-	/* buf_getint(ses.payload);*/
-
+	buf_getint(ses.payload);
 	return;
 
 error:
 	dropbear_exit("no matching algo %s", erralgo);
-
 }
 
+/* Send our list of algorithms we can use */
 void send_msg_kexinit() {
 
 	CHECKCLEARTOWRITE();
